###
### Instance Variables and Fields in Reference Classes
###
### Martin Schedlbauer, PhD
### Northeastern University
###

####################################################################
###
### Define ClassA
###
####################################################################

ClassA <- setRefClass("ClassA", 
                      
                      # fields/attributes
                      fields = list(
                        ID = "integer",
                        fieldA = "numeric",
                        refB = "ANY"),
                      
                      # methods/member functions
                      methods = list(
                        
                        # Constructor -- called upon object instantiation
                        initialize = function(id = 0, fa = "", rb = NULL) {
                          ID <<- id
                          fieldA <<- fa
                          refB <<- rb
                        },
                        
                        # Method: memFuncA()
                        memFuncA = function(aB) {
                          cat("within memFuncA on ClassA instance ID: ", ID, "\n")
                          
                          # access instance variable
                          cat("fieldA:", fieldA, "\n")
                          
                          # access members of argument 'aB'
                          cat("fieldB of 'aB' arg:", aB$fieldB, "\n")
                          
                          # access member 'refB'
                          cat("fieldB of 'refB' member:", refB$fieldB, "\n")
                          
                          # local variable of same name as member
                          ID <- 5555
                          cat("local var ID:", ID, "\n")
                          
                          # change field rather than local variable
                          ID <<- 3333L
                          cat("local var ID:", ID, "\n")
                          cat("local var ID:", .self$ID, "\n")
                          
                          return (TRUE)
                        }
                      ))

####################################################################
###
### Define ClassB
###
####################################################################

ClassB <- setRefClass("ClassB", 
                      fields = list(
                        ID = "integer",
                        fieldB = "character"),
                      
                      methods = list(
                        getFieldB = function() {
                          return (fieldB)
                        }
                      ))


# create two instances of ClassB
b1 <- ClassB(ID = 77L, fieldB = "instance one of ClassB")
b2 <- ClassB(ID = 88L, fieldB = "instance two of ClassB")

# instantiate ClassA and set member field to ClassB
a <- ClassA(11L, 100, b1)

# call member function on 'a' passing 'b2' as argument
a$memFuncA(b2)

# when instantiating with the method new(), the initialize()
# method (constructor) is automatically invoked
aa <- ClassA$new()
