##
## Code to accompany tutorial/recitation for l-6-122
##

## define a new class "Instructor" using the reference class system

Instructor <- setRefClass(
  "Instructor", 
  fields = list(
    iid="numeric", 
    name="character", 
    rank="character")
)

## instantiate a new instance of the class Instructor, i.e., create an
## object of type Instructor

i1 <- Instructor(iid = 132, name = 'Jeff Alden', rank = 'PT')

print(i1)

# read a field's value
n <- i1$name

# update a field's value
i1$name <- 'Jeffrey Alden'

print(i1)

## objects as references

# this copies the reference; i99 and i1 now "point" to the same object in memory
i99 <- i1

i99$name <- 'Xin Wang'

print(i1)

# we need to make a copy if we actually want a new copy that has the same
# content as an existing object

i199 <- i1$copy()

# modifying i199 does not modify i1
i199$name <- 'Susan Wollaston'

print(i1)

## methods/functions on objects

Instructor <- setRefClass("Instructor", 
    fields = list(iid="numeric", 
                  name="character", 
                  rank="character",
                  salary="numeric"
    ),
    methods = list(
      getMonthlySalary = function() {
        return (.self$salary / 12)
      },
      
      applyRaise = function(merit) {
        .self$salary <- .self$salary * (1 + merit)
      }
    ))

i <- Instructor(iid = 11, name = 'Kaleb Ahmad', 
                rank = 'FT-Associate', salary = 200000)

i$getMonthlySalary()

m.bef <- i$getMonthlySalary()
i$applyRaise(0.045)

m.aft <- i$getMonthlySalary()


## Inheritance (implementation of "generalization")

Person <- setRefClass("Person", 
    fields = list(pid="numeric", 
                  name="character",
                  yob = "numeric"),
    methods = list(
      getAge = function() {
        currYear <- as.numeric(format(Sys.time(), "%Y"))
        return (currYear - yob)
      }
    ))

Instructor <- setRefClass("Instructor", 
    contains = "Person",
    fields = list(rank="character",
                  salary="numeric"
    ),
    methods = list(
      getMonthlySalary = function() {
        return (salary / 12)
      },
      
      applyRaise = function(merit) {
        salary <<- salary * (1 + merit)
      }
    ))

anInstructor <- Instructor(pid = 100, 
                           name = 'Raj Metha', 
                           rank = 'FT-Full',
                           yob = 1968,
                           salary = 182972)

anInstructor$getMonthlySalary()
anInstructor$getAge()









## constructors: an alternative way to instantiate objects

Member <- setRefClass("Member", 
                      fields = list(
                        mID = "numeric", 
                        name = "character",
                        yearJoined = "numeric"),
                      
                      methods = list(
                        initialize = function(name, year) {
                          .self$name <- name
                          .self$yearJoined <- year
                          .self$mID <- 99
                        }
                      ))


# create an instance with implicit initialization
aMember <- Member$new('Liz Chao', 2023)

print(aMember)


